/*-
 * Copyright (c) 2011-2012       Scott Ringwelski <sgringwe@mtu.edu>
 *
 * Originally Written by Scott Ringwelski for BeatBox Music Player
 * BeatBox Music Player: http://www.launchpad.net/beat-box
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#if HAVE_UNITY
#if HAVE_DBUSMENU
using GLib;

public class BeatBox.UnityIntegration : GLib.Object {
	const string PLAYLIST_ID = "/org/gnome/BeatBox/Playlist/%d";
	
	LibraryManager lm;
	LibraryWindow lw;
	Unity.LauncherEntry entry;
	
	Unity.MusicPlayer player;
	Gee.HashMap<PlaylistInterface, Unity.Playlist> playlists;
	
	Dbusmenu.Menuitem quicklist;
	Dbusmenu.Menuitem previous;
	Dbusmenu.Menuitem playpause;
	Dbusmenu.Menuitem next;
	Dbusmenu.Menuitem equalizer;
	Dbusmenu.Menuitem preferences;
	
	public UnityIntegration(LibraryManager lm, LibraryWindow lw) {
		this.lm = lm;
		this.lw = lw;
	}
	
	public bool initialize() {
		entry = Unity.LauncherEntry.get_for_desktop_file("beatbox.desktop");
		if(entry == null)
			return false;
		
		quicklist = new Dbusmenu.Menuitem ();
		previous = new Dbusmenu.Menuitem ();
		playpause = new Dbusmenu.Menuitem ();
		next = new Dbusmenu.Menuitem ();
		equalizer = new Dbusmenu.Menuitem ();
		preferences = new Dbusmenu.Menuitem ();
		previous.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Previous"));
		playpause.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Play"));
		next.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Next"));
		equalizer.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Equalizer"));
		preferences.property_set (Dbusmenu.MENUITEM_PROP_LABEL, _("Preferences"));
		previous.item_activated.connect(previous_activated);
		playpause.item_activated.connect(playpause_activated);
		next.item_activated.connect(next_activated);
		equalizer.item_activated.connect(equalizer_activated);
		preferences.item_activated.connect(preferences_activated);
		quicklist.child_append (previous);
		quicklist.child_append (playpause);
		quicklist.child_append (next);
		var sep = new Dbusmenu.Menuitem();
		sep.property_set(Dbusmenu.MENUITEM_PROP_TYPE, Dbusmenu.CLIENT_TYPES_SEPARATOR);
		quicklist.child_append (sep);
		quicklist.child_append (equalizer);
		quicklist.child_append (preferences);
		entry.quicklist = quicklist;
		
		lm.progress_notification.connect(progress_notification);
		lm.file_operations_started.connect(file_operations_started);
		lm.file_operations_done.connect(file_operations_done);
		
		player = new Unity.MusicPlayer("beatbox.desktop");
		player.export();
		
		// Add all playlists to the sound menu
		playlists = new Gee.HashMap<PlaylistInterface, Unity.Playlist>();
		foreach(var p in lm.playlists())
			playlist_added(p);
		foreach(var sp in lm.smart_playlists())
			playlist_added(sp);
		
		player.next.connect(lw.nextClicked);
		player.previous.connect(lw.previousClicked);
		player.play_pause.connect(lw.playClicked);
		player.activate_playlist.connect(playlist_activated);
		
		lw.playPauseChanged.connect(playing_changed);
		
		lm.media_played.connect(media_played);
		lm.playlist_added.connect(playlist_added);
		lm.playlist_removed.connect(playlist_removed);
		
		return true;
	}
	
	void previous_activated(uint object) {
		lw.previousClicked();
	}
	
	void playpause_activated(uint object) {
		lw.playClicked();
	}
	
	void next_activated(uint object) {
		lw.nextClicked();
	}
	
	void equalizer_activated(uint object) {
		lw.editEqualizerClick();
	}
	
	void preferences_activated(uint object) {
		lw.editPreferencesClick();
	}
	
	void playlist_activated(ObjectPath path) {
		string playlist_id_prefix = PLAYLIST_ID.replace("%d", "");
		string id_s = path.replace(playlist_id_prefix, "");
		int id = int.parse(id_s);
		
		Object o;
		if(id >= 1000)
			o = lm.smart_playlist_from_id(id - 1000);
		else
			o = lm.playlist_from_id(id);
		
		if(o == null) {
			warning("Selected playlist had invalid path %s and could not be found in db\n", path);
			return;
		}
		
		string name;
		if(o is Playlist)
			name = ((Playlist)o).get_name();
		else
			name = ((SmartPlaylist)o).get_name();
		
		// Start playing that list
		Gtk.TreeIter? iter = lw.sideTree.getIterFromObject(lw.sideTree.playlists_iter, o);
		if(iter != null) {
			lw.sideTree.setSelectedIter(lw.sideTree.convertToFilter(iter));
			ViewWrapper vw = (ViewWrapper)lw.sideTree.getWidget(iter);
			vw.play_first_media();
		}
		else {
			warning("Selected playlist was valid but associated view wrapper could not be found to play media\n");
			return;
		}
	}
	
	void file_operations_started() {
		entry.progress_visible = true;
	}
	
	void file_operations_done() {
		entry.progress_visible = false;
	}
	
	void progress_notification(string? message, double progress) {
		if(entry == null)
			return;
		
		entry.progress = progress;
	}
	
	void playing_changed() {
		string lbl = (lm.playing) ? _("Pause") : _("Play");
		playpause.property_set (Dbusmenu.MENUITEM_PROP_LABEL, lbl);
		
		player.playback_state = lm.playing ? Unity.PlaybackState.PLAYING : Unity.PlaybackState.PAUSED;
	}
	
	void media_played(Media m, Media? old) {
		Unity.TrackMetadata meta = new Unity.TrackMetadata();
		meta.title = m.title;
		meta.artist = m.artist;
		meta.album = m.album;
		meta.art_location = File.new_for_path(m.getAlbumArtPath());
		player.current_track = meta;
	}
	
	void playlist_added(PlaylistInterface p) {
		if(playlists.get(p) != null) {
			warning("Playlist was added, but is already in soundmenu. Not adding duplicate\n");
			return;
		}
		
		int id = p.get_rowid();
		if(p is SmartPlaylist)
			id += 1000;
		
		Unity.Playlist up = new Unity.Playlist(PLAYLIST_ID.printf(id));
		up.name = p.get_name();
		up.icon = Icons.PLAYLIST.get_gicon();
		
		playlists.set(p, up);
		player.add_playlist(up);
		p.changed.connect( () => {
			playlist_changed(p);
		});
	}
	
	void playlist_removed(PlaylistInterface p) {
		var up = playlists.get(p);
		if(up == null) {
			warning("Playlist was removed, but was not in unity playlist table. Cannot remove from soundmenu\n");
			return;
		}
		
		player.remove_playlist(up);
		playlists.unset(p);
	}
	
	void playlist_changed(PlaylistInterface p) {
		int id = p.get_rowid();
		if(p is SmartPlaylist)
			id += 1000;
		
		player.edit_playlist_name(PLAYLIST_ID.printf(id), p.get_name());
	}
}

#endif
#endif
