/*-
 * Copyright (c) 2011-2012       Scott Ringwelski <sgringwe@mtu.edu>
 *
 * Originally Written by Scott Ringwelski for BeatBox Music Player
 * BeatBox Music Player: http://www.launchpad.net/beat-box
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

using Gee;
using Gtk;

public abstract class BeatBox.GenericGrid : FastGrid {
	protected LibraryManager lm;
	protected LibraryWindow lw;
	protected ViewWrapper parent_wrapper;
	
	// For shuffle
	protected int old_sort_col;
	protected SortType old_sort_dir;
	
	protected TreeViewSetup tvs;
	protected int relative_id;
	protected bool is_current_list;
	
	protected bool scrolled_recently;
	protected bool dragging;
	
	protected GLib.Icon playing_icon;
	protected GLib.Icon completed_icon;
	protected GLib.Icon saved_locally_icon;
	protected GLib.Icon new_podcast_icon;
	
	// To select which columns are showing
	protected Gtk.Menu columnChooserMenu;
	
	public signal void import_requested(LinkedList<Media> to_import);
	
	private const string WIDGET_STYLESHEET = """
        .GenericGrid {
            
        }
    """;
	
	private const int ITEM_PADDING = 3;
	private const int ITEM_WIDTH = Icons.ALBUM_VIEW_IMAGE_SIZE;
	private const int SPACING = 12;
	
	public GenericGrid(LibraryManager lm, TreeViewSetup tvs, int rel_id, GLib.Object default_value) {
		base(default_value);
		
		this.lm = lm;
		this.lw = lm.lw;
		this.tvs = tvs;
		this.relative_id = rel_id;
		
		playing_icon = Icons.MEDIA_PLAY_SYMBOLIC.get_gicon ();
		completed_icon = Icons.PROCESS_COMPLETED.get_gicon ();
		saved_locally_icon = new GLib.ThemedIcon.with_default_fallbacks (Gtk.Stock.SAVE);
		new_podcast_icon = Icons.NEW_PODCAST.get_gicon ();
		
		// Change background color
		var style_provider = new CssProvider();

        try  {
            style_provider.load_from_data (WIDGET_STYLESHEET, -1);
        } catch (Error e) {
            stderr.printf ("\nGenericGrid: Couldn't load style provider.\n");
        }

        get_style_context().add_class("GenericGrid");
        get_style_context().add_provider(style_provider, STYLE_PROVIDER_PRIORITY_APPLICATION);
		
		item_width = ITEM_WIDTH;
		item_padding = ITEM_PADDING;
		spacing = 2;
        margin = 12;
		
		// drag source
		TargetEntry te = { "text/uri-list", TargetFlags.SAME_APP, 0};
		drag_source_set(this, Gdk.ModifierType.BUTTON1_MASK, { te }, Gdk.DragAction.COPY);
		//enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK, {te}, Gdk.DragAction.COPY);
		
		//vadjustment.value_changed.connect(view_scroll);
		//row_activated.connect(row_activated_signal);
		//rows_reordered.connect(updateTreeViewSetup);
		lm.current_cleared.connect(current_cleared);
		//lm.media_played.connect(media_played);
		//lm.medias_updated.connect(medias_updated);
		size_allocate.connect(size_allocate_signal);
	}
	
	void size_allocate_signal(Allocation all) {
		Idle.add( () => {
			recalculate_columns(all.width);
			
			return false;
		});
	}
	
	void recalculate_columns(int width) {
		int columns = 0;
			int rem = width;
		do {
			rem -= ITEM_WIDTH;
			++ columns;
		} while(rem > 100);
		
		columns -= 1;
		if(columns <= 0)
			columns = 1;
		
		if(columns > 1) {
			rem += SPACING * 6;
			column_spacing = rem / ((columns == 1) ? columns : (columns - 1));
		}
		else {
			column_spacing = 0;
			item_padding = 0;
			spacing = 0;
		}
		
		set_columns(columns);
	}
	
	public void set_parent_wrapper(ViewWrapper parent) {
		this.parent_wrapper = parent;
	}
	
	public abstract void update_sensitivities();
	
	/** TreeViewColumn header functions. Has to do with sorting and
	 * remembering column widths/sort column/sort direction between
	 * sessions.
	**/
	protected abstract void updateTreeViewSetup();
	
	
	void current_cleared() {
		is_current_list = false;
	}
	
	/***************************************
	 * Simple setters and getters
	 * *************************************/
	public void set_hint(ViewWrapper.Hint hint) {
		tvs.set_hint(hint);
	}
	
	public ViewWrapper.Hint get_hint() {
		return tvs.get_hint();
	}
	
	public void set_relative_id(int id) {
		this.relative_id = id;
	}
	
	public int get_relative_id() {
		return relative_id;
	}
	
	public bool get_is_current_list() {
		return is_current_list;
	}
}
